######################################################################
# Contrary to what it looks like, this should _not_ be treated like a
# general Perl module ;-)
# 
# This is just an Embperl object module, meant to handle the SmbGate
# web-based interaction.
#
# Very short walk-through (you should really be familiar with embperl
# if you want to do anything else in here): 
#
# - Whenever a page is called in the Embperl-requested area, this
#   module gets run first
# - Then, we go through _base.epl, a general layout for the whole 
#   system
# - _base.epl calls several other helper modules to build the base 
#   page
# - Finally, the user-requested file is executed (also through embperl)
#   and the requests get sent back to the browser
#
# The functions in here should be decently commented and easy to follow 
# through. In case of any other question, contact me (Gunnar Wolf 
# <gwolf@gwolf.org>).
######################################################################
# Just for the record: This system is Free Software. For details on its
# copyright and licensing, please refer to the README file in the root of
# this distribution.
use strict;
use warnings;

use Filesys::SmbClient;
use YAML;

use SmbGate;
use SmbGate::Embperl;

our @ISA = qw(Embperl::App);

sub init {
    my ($self);
    $self = shift;

    # Errors and messages should always be arrayrefs, even if empty
    $udat{err} = [] unless $udat{err};
    $udat{msg} = [] unless $udat{msg};
    $epreq->{warnings} = SmbGate::Embperl::Warnings->new($udat{err}, $udat{msg});

    $self->baseconf;
    $self->expire_session;
    $epreq->{conf} = $self->{baseconf};

    # Set up the base SmbGate object
    $epreq->{smb} = SmbGate->new($epreq->{conf}->max_filesize);

    $epreq->{webclient} = SmbGate::Embperl::WebClient->new($epreq,
							   \%http_headers_out,
							   \%udat,
							   $epreq->{conf}->allow_debug);
    $epreq->{webclient}->content_type('text/html'); # Can be changed later

    # Validate user if we get login/passwd
    if ($fdat{login} and $fdat{passwd}) {
	# Just pass them on to the session - We will check them right away.
	# We store both login _and_ password in our session, as we will
	# need both to start future Samba sessions
	$udat{login} = $fdat{login};
	$udat{passwd} = $fdat{passwd};
    }

    if ($udat{login} and $udat{passwd}) {
	$epreq->{smb}->login("smb://iiec/$udat{login}",
			     $udat{login}, $udat{passwd}) or
	    $epreq->{warnings}->add_err("Usuario/contrase&ntilde;a inv&aacute;lidos");

    }

    # Sorry, ma'am. No valid user no play.
    unless ($epreq->{smb}->is_logged_in or 
	    $epreq->param->filename =~ m!/(?:doc/\w*|login).html$!) {
	$epreq->{webclient}->redirect('login.html');
    }

    # Initialize the HTML helper module (this has to happen after the user is
    # logged in)
    $epreq->{html} = SmbGate::Embperl::HtmlHelper->new(user => $udat{login},
				           base => $epreq->{conf}->base_url);

    # Debug mode? Take care of it here. Leave authorization to the called
    # methods.
    $epreq->{webclient}->set_debug if $fdat{set_debug};
    $epreq->{webclient}->unset_debug if $fdat{unset_debug};

    0;
}

sub baseconf {
    my ($self, $filename, $instance);
    $self = shift;
    $instance = $ENV{SMBGATE_INSTANCE} || 'smbgate';
    $filename = $ENV{SMBGATE_YAMLCONF} || '/etc/smbgate.yaml';

    $self->{baseconf} = SmbGate::Embperl::Baseconf->new($instance, $filename)
	or die "Could not get system configuration";
    return 1;
}

sub expire_session {
    # Expire the person_id from the session data if too much time has passed
    # since we last saw it
    my ($self, $expire_time, $now);
    $self = shift;

    # Default ession expiration time: 10 minutes.
    # A negative value for the session expiration time means never to expire
    $expire_time = $self->{baseconf}->session_timeout;
    # ...But we need seconds
    $expire_time *= 60;

    $now = time;
    $udat{expire} ||= 0;
    if ($expire_time >= 0 and $udat{expire} < $now) {
	delete $udat{login};
	delete $udat{passwd};
	if ($udat{expire}) {
	    $epreq->{warnings}->add_msg("Su sesi&oacute;n ha expirado.");
	}
    }

    $udat{expire} = $now + $expire_time;

    return 1;
}

sub _human_size {
    my ($amount, $res, @max);
    $amount = shift;

    @max = ([2**30, 'GB'],
	    [2**20, 'MB'],
	    [2**10, 'KB'],
	    [1, 'Bytes']);

    for my $scale (@max) {
	next if $amount/1024 > $scale->[0];
	$res = $amount / $scale->[0];
	$res =~ s/^(\d+\.\d).*/$1/;
	$res .= " $scale->[1]";
    }

    return $res;
}

1;
