#!/usr/bin/perl
#
# This object is not worth too much to the SmbGate system - I just left it
# around as it is a good way of understanding Filesys::SmbClient.
use strict;
use warnings;
use YAML;
use Filesys::SmbClient;
use Getopt::Long;

my ($smb, $tree, %conf);

%conf = ();

GetOptions(\%conf, 'user=s', 'passwd=s', 'server=s', 'debug:i', 'share:s',
	   'quiet')
    or die usage();

$conf{server} = 'iiec' unless $conf{server};
$conf{share} = "//$conf{server}/$conf{user}" unless $conf{share};
$conf{debug} = 0 unless $conf{debug};

$smb = Filesys::SmbClient->new(username => $conf{user},
			       password => $conf{passwd},
#			       workgroup => 'ECONOMICAS',
			       debug => $conf{debug})
    or die "Error setting up SMB object";

$tree = tree($smb, $conf{share});

print YAML::Dump $tree unless $conf{quiet};

sub tree {
    my ($smb, $base, $dir, %tree);
    $smb = shift;
    $base = shift;

    debug(5, "Opening smb:$base");
    unless ($dir = $smb->opendir("smb:$base")) {
	debug(0, "Error opening directory $base:\n$!\n");
    }

    while (my $entry = $smb->readdir_struct($dir)) {
	next if $entry->[1] eq '.' or $entry->[1] eq '..';
	if ($entry->[0] == SMBC_DIR) {
	    my $subdir = "$base/$entry->[1]";
	    debug(4, "Entering directory: $subdir");
	    $tree{$entry->[1]} = tree($smb, "$base/$entry->[1]");
	} elsif ($entry->[0] == SMBC_FILE) {
	    $tree{$entry->[1]} = '';
	    debug(7, "File: $entry->[1]");
	} else {
	    debug(3, "Unknown object: $entry->[1]");
	}
    }
    $smb->closedir($dir);

    return \%tree;
}

sub usage {
    return "Moron!"
}

sub debug {
    my ($level, $txt);
    $level = shift;
    $txt = shift;
    warn $txt if $level <= $conf{debug};
}
